#!/bin/sh
#
#  install.sh
#
#  Install Virtuoso Lite & Enterprise editions
#
#  Version: 2.1.3
#  

#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#  
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#
#  To learn more about this product, or any other product in our
#  portfolio, please check out our web site at:
#
#      http://www.openlinksw.com
#
#  or contact us at:
#
#      general.information@openlinksw.com
#
#  If you have any technical questions, please contact our support
#  staff at:
#
#      technical.support@openlinksw.com
#


#
#Set up the script
#

umask 022

#
#  Installation directory
#
PREFIX=`pwd`

#
#  Setup the PATH
#
PATH="$PREFIX/bin:$PREFIX/demo:$PREFIX/db:".:$PATH
export PATH

#
#  Test for -n or \c in echo command
#
if (echo "testing\c") | grep c >/dev/null; then
    echo_n=-n
    echo_c=
else
    echo_n=
    echo_c='\c'
fi

#
#  Check whether to use uncompress or gunzip
#
UNZIP=uncompress
SUFFIX=Z
IFS="${IFS=   }"; SAVE_IFS="$IFS"; IFS=":"
for dir in $PATH
  do
  if [ -x $dir/gunzip ]
      then
      UNZIP=$dir/gunzip
      SUFFIX=gz
      break
  fi
done
IFS="$SAVE_IFS"


#
# Useful functions to be having
#

overwritewarn() {
# issue a warning about data entry that could cause lost functionality when
# upgrading over an existing instance
  if [ "z$INSTANCE" != "z" ]
  then

  echo '* Warning: you could overwrite configuration or data used by existing'
  echo '*          instances, thereby impacting functionality'
  fi
}

locatelib() {
# Attempt to file $1 in a set of standard locations, or using locate(1)
    requiredfile="$1"

    [ -x "/bin/locate" -o -x "/usr/bin/locate" ] && locatedfiles=`locate "$requiredfile" 2> /dev/null`

    [ -x "/bin/whereis" -o -x "/usr/bin/whereis" ] && whereisfiles=`whereis "$requiredfile" 2> /dev/null`

    deflocationfiles=`find /usr/lib/ /usr/local/lib/ /opt/j* /usr/j* /usr/local/j* . /Library/System -name "$requiredfile".\* 2> /dev/null`

    RETVALUE="$locatedfiles $whereisfiles $deflocationfiles"
}


finddefaultserver() {
# Make a suggestion as to the most feature-full server possible

    echo "Checking for installed optional components..."

    SuggestingJava=0
    SuggestingMono=0

    locatelib "libjvm"
    RETVALUE=`echo "$RETVALUE" | sed 's/ *//g'`
    if [ \( "$INSTALLINGADB" -gt 0 \) -o \( ! -z "$RETVALUE" \) ] 
        then
        SuggestingJava=1
    fi

    locatelib "corlib"
    RETVALUE=`echo "$RETVALUE" | sed 's/ *//g'`
    if [ ! -z "$RETVALUE" ]
        then
        SuggestingMono=1
    fi

    RETVALUE=2
    if [ \( $SuggestingMono -eq 1 \) -a \( $SuggestingJava -eq 1 \) ]
        then
        RETVALUE=8
    else
        if [ $SuggestingMono -eq 1 ]
            then
            RETVALUE=6
        fi
        if [ $SuggestingJava -eq 1 ]
            then
            RETVALUE=5
        fi
    fi        


    if [ \( "$RETVALUE" -gt 5 \) -a \( $ONLINUX = 0 \) ]
        then
        RETVALUE=5
    fi

    if [ "$ONBSD" -gt 0 ] 
        then
        RETVALUE=3
        SuggestingJava=0
        SuggestingMono=0
    fi
}

checkgclib() {
#check if mono binary is choosen to break if no libgc; otherwise warning
    stop_install=0
    case $1 in
  *-mono-*)
            stop_install=1
            ;;
        *)
            ;;
    esac
    if [ -f "$MONOPATH/bin/mcs" ]
        then
        LD_LIBRARY_PATH="$MONOPATH/lib:$LD_LIBRARY_PATH"
        MONO_PATH="$MONOPATH/lib"
        MONO_GAC_PATH="$MONOPATH/lib"
        export LD_LIBRARY_PATH MONO_PATH MONO_GAC_PATH
        "$MONOPATH/bin/mcs" -? 2> /dev/null 1> /dev/null
        if [ $? != 0 ]
            then
            echo ""
            echo "** Alert:"
            echo "** The package combination you have chosen requires a"
            echo "** MONO library to be installed on your system."
            echo "** The MONO library is available for download from"
            echo "** the MONO Web site (http://www.go-mono.com)."
            if [ $stop_install == 0 ]  
                then
                echo "** You may press ^C to exit this installer and install"
                echo "** the MONO library or proceed by pressing "
                echo "** Enter."
            fi
            echo ""
            if [ $stop_install != 0 ]
                then
                exit -1
            else
                read abort
            fi
        fi
    fi

}

chooseserver() {
#Ask the user for a choice of server, given our sensible default

    echo "Please choose which Virtuoso server you wish to install:"
    echo ""

    servers=""
    javawarning=0
    c=0
    for i in `/bin/ls -1 bin/virtuoso*-t | grep -v opmono | grep -v sample | sort -r`
      do
      c=`expr 1 + $c`
      servers="$servers $i"
      desc=`echo $i | sed 's/bin.virtuoso-/Universal Server /; 
                           s/iodbc/with iODBC/;
                           s/-javavm/, Java/;
                           s/-php/, PHP/;
                           s/-mono-cli/, CLR Runtime Hosting with Mono/;
                           s/-t$/ support/'`
      echo "  $c) $desc"
      eval serverbinary$c=$i
      echo $i | grep -i 'java' > /dev/null && javawarning=1
    done

    MyDefault=$c

    if [ "$javawarning" -gt 0 ]
    then
        echo ""
        echo "Note: if you install a java-enhanced Virtuoso server, you must have a working"
        echo "JVM installation on this server."
    fi

    overwritewarn

    echo ""
    echo "[$MyDefault] :"

    read server

    server=`echo $server | sed 's/^[()[]]//; s/[()[]]$//; s/^(.).*$/$1/'`

    if [ -z "$server" ]
        then
        server=$MyDefault
    fi

    varname="SERVER=\$serverbinary$server"
    eval $varname
    SERVER=`echo $SERVER|sed 's/^bin.//'`
}

findcomponent() {
    servertype=$1
    requiredfile=$2
    desc=$3

    echo "Attempting to locate $desc ($requiredfile)..."

    locatelib $requiredfile
    
    foundfiles="/usr/local/lib/$requiredfile $RETVALUE"

    echo $foundfiles | egrep 'server' > /dev/null
    if [ ! $? ]
        then
        usingfile=`echo $foundfiles | xargs -n 1 echo | egrep 'server' | tail -1`
    else
        usingfile=`echo $foundfiles | xargs -n 1 echo | tail -1`
    fi

    suggestedfile=$usingfile

    suggesteddir=`dirname $suggestedfile`

    echo ''
    echo "Suggested file locations: "
    echo $foundfiles | xargs -n 1 dirname | xargs -n 1 echo
    echo ''

    echo "Please select one of the above locations or press Enter to use"
    echo "the default ($suggesteddir):"

    read RETVALUE

    if [ -z "$RETVALUE" ]
        then
        RETVALUE=$suggesteddir
    fi

#    echo "Using value $suggesteddir"
}

#
# Things to do when installing a component including a server
#

shutdown_server () {
  ../bin/isql $1 dba $dbapw1 -K > /dev/null 2>&1
}

vadinstall() {
#Function to install VAD's 

    if [ "z$3" = "zDefault" ]
    then
  text="Default Database"
  ini_file=virtuoso
  server_port=$DBSQLPORT
    else
  text="Demo Database"
  ini_file=demo
  server_port=$DEMOSQLPORT
    fi

    echo "Install " $1 "on" $text
    echo "([Y]/N):"

    vad_inst="yes"
    read vad_inst
    
    [ -z "$vad_inst" ] && vad_inst="yes"
    echo "$vad_inst" | grep -i '^y' > /dev/null && vad_inst="yes"

    if [ "$vad_inst" = "yes" ]
  then
      echo "Installing $1 on $text. Please wait..."
  if [ ! -f $ini_file.lck ];
  then
      ../bin/virtuoso-iodbc-t +configfile $ini_file +wait > /dev/null 2>&1
      if test $? -ne 0
      then
        echo
        echo "** Error starting Virtuoso DBMS Demo server"
        echo
      fi
      while [ "z$stat" != "z" ]
      do
    sleep 5
    stat=`netstat -an | grep "[\.\:]$server_port " | grep LISTEN`
      done
  fi

  ../bin/isql $server_port dba $dbapw1 EXEC="DB.DBA.INSTALLER_VAD ('$2')" > /dev/null 2>&1

  if [ -f ../vad/install.res ]
  then
      cat ../vad/install.res > res_install
  fi

  if [ "z$res_install" != "z" ]
  then
      echo "Problem:"
      echo $res_install
      echo "= = = = =" 
  fi
    
  rm -rf ../vad/install.res

    fi
}

chpasswd() {
#Function to change a dba or dav password for a given instance
    u=$1
    instance=$2
    desc=$3

    pw1="error1" 
    pw2="error2"

    saveterm=`stty -g`
    stty -echo

    overwritewarn

    while [ "$pw1" != "$pw2" ]
      do
      echo ""
      echo "Please enter a replacement for the $instance"
      echo "$desc ($u) account password (twice) :"
      read pw1 

      if [ -z "$pw1" ]
          then 
          pw1="error1"
      fi

      echo "Confirm Password : $echo_c"
      read pw2 
    done

    stty $saveterm
}


whichperl()
{
    RETVALUE=""

    echo "Attempting to locate perl library..."

    locatelib libperl

    foundfiles="$RETVALUE"
    usingfile=`echo $foundfiles | xargs -n 1 echo | tail -1`
    suggestedfile=$usingfile
    suggesteddir=`dirname $suggestedfile`
    RETVALUE=""

    perlout=`perl -e 'print "Problem\n" if( $] < 5.008);'`

    PERLSAFE=""
    PERLINISAFE=""

    if [ "$perlout" = "Problem" ]
        then

        echo 'Warning: the version of Perl in your PATH is older than the default library'
        echo 'we ship (5.8.0). Please either specify your own version of the library or'
        echo 'update your version of Perl. For more details, see README-perl.txt in the'
        echo 'installation directory.'
        PERLSAFE="#"
        PERLINISAFE=";"
    fi


    echo ''
    echo "Suggested file locations: "
    echo $foundfiles | xargs -n 1 dirname | xargs -n 1 echo
    echo ''

    echo "Please select one of the above locations. Note that the libperl.so must"
    echo "have been compiled with -Dusemultiplicity; if you are unsure of this, use"
    echo "the one shipped with Virtuoso (last entry in the list)."
    echo "Default: $suggesteddir"

    read RETVALUE

    if [ -z "$RETVALUE" ]
    then  
        RETVALUE=$suggesteddir
  perl_path="$RETVALUE/perl"  
    fi
}

enable_plugin() 
{
    _replace=";LoadN"

    _so_file="$1"
    _qline1="$2"
    _qline2="$3"
    _ask=$4
    _reptext=$5
    plugin_no=$6

    if [ -f  "$_so_file" ] 
    then

        echo ""
        echo "$_qline1"
        echo "$_qline2"

        echo "(Y/[N]):"

        add_ext="no"
        read add_ext
        
        [ -z "$add_ext" ] && add_ext="no"
        echo "$add_ext" | grep -i '^y' > /dev/null && add_ext="yes"

        if [ "$add_ext" = "yes" ]
        then
            _replace="Load$plugin_no"
            echo "$ask"
        else
            _replace=";Load$plugin_no"
        fi
        for f in demo/demo.ini db/virtuoso.ini
        do
            sed -e "s/$_reptext/$_replace/g" "$f" > "$f.new" && mv "$f.new" "$f"
        done
    fi
}

enable_plugins()
{
    enable_plugin \
      "hosting/hosting_perl.so" \
  "Do you want to install support for scripting in Perl?" \
  "(Note: if you wish to build this yourself, please see README-PERL.txt)" \
  "Adding Perl to ini-file [Plugins] section" \
  "^Load1" \
        1

    enable_plugin \
      "hosting/hosting_python.so" \
  "Do you want to install support for scripting in Python?" \
  "(Note: if you wish to build this yourself, please see README-PYTHON.txt)" \
  "Adding Python to ini-file [Plugins] section" \
  "^Load2" \
        2

    enable_plugin \
      "hosting/hosting_ruby.so" \
  "Do you want to install support for scripting in Ruby?" \
  "(Note: if you wish to build this yourself, please see README-RUBY.txt)" \
  "Adding Ruby to ini-file [Plugins] section" \
  "^Load3" \
        3

    enable_plugin \
      "hosting/wikiv.so" \
  "Do you want to install support for Wiki ?" \
  "" \
  "Adding Wiki plugin to ini-file [Plugins] section" \
  "^Load4" \
        4

    enable_plugin \
      "hosting/mediawiki.so" \
  "Do you want to install support for MediaWiki compatible syntax for Wiki ?" \
  "" \
  "Adding MediaWiki plugin to ini-file [Plugins] section" \
  "^Load5" \
        5

    enable_plugin \
      "hosting/im.so" \
  "Do you want to install support for Image Magick ?" \
  "" \
  "Adding Image Magick plugin to ini-file [Plugins] section" \
  "^Load6" \
        6

    enable_plugin \
      "hosting/wbxml2.so" \
  "Do you want to install support for WBXML ?" \
  "" \
  "Adding WBXML2 plugin to ini-file [Plugins] section" \
  "^Load7" \
        7
}

HOSTNAME=`hostname | sed 's/\..*$//'`

echo Installation started on $HOSTNAME

ONLINUX=0
uname | grep Linux > /dev/null && ONLINUX=1
ONBSD=0
uname | grep FreeBSD > /dev/null && ONBSD=1

#
#Build up a list of files to unpack
#

FILES=`ls -1  *.taz *.tgz *.tar.gz *.tar.Z 2> /dev/null | egrep -i \
    'enter|lite|client|docs|demo|tutorial|admin|samp|mono|im' | xargs echo`

echo ""

#
#Check for strange package combinations
#

echo ""
echo "Checking requested components"

echo $FILES | egrep -i "enter.*(lite|demo)" > /dev/null

if [ $? -eq 0 ]
    then
    echo ""
    echo "** Warning:"

    echo "The Enterprise Installer already contains all the other components;"
    echo "if you wish for this to be the only file installed, press Enter now"
    echo "to continue. Otherwise, please abort the script with ^C and move the"
    echo "Enterprise installer out of this directory."

    read abort

    FILES=`echo $FILES | sed -e 's/lite[^ ]* / /g; s/demo[^ ]* / /g'`
fi

if [ -z "$FILES" ]
    then
    echo ""
    echo "** Error:"
    echo "No packages found. Are you running this install.sh from a "
    echo "directory in which you have unpacked the downloaded tarball?"
    echo ""
    echo "Quitting..."
    
    exit -1
fi

#
#Build up variables for conditional tests later
#
INSTALLINGADB=0
INSTALLINGCLIENT=0
INSTALLINGSUNDRIES=0
echo "$FILES" | egrep -i "enter|lite" > /dev/null && INSTALLINGADB=1
echo "$FILES" | egrep -i "client|demo" > /dev/null && INSTALLINGCLIENT=1
echo "$FILES" | egrep -i "mono" > /dev/null && INSTALLINGMONOBITS=1
echo "$FILES" | egrep -i "admin|docs|tutorial|samples|im" > /dev/null && \
    INSTALLINGSUNDRIES=1

#
#Sanity checks
#

if [ $INSTALLINGSUNDRIES -gt 0 ]
    then

    if [ $INSTALLINGADB -eq 0 ]
        then

        echo "Warning. The package combination you have chosen requires a"
        echo "Virtuoso server package in order to be used fully, either"
        echo "Enterprise or Lite. You should download one from"
        echo "<http://www.openlinksw.com/> and restart this install script. "
        echo "Alternatively, you can press Enter now to continue, but the"
        echo "installation may leave an incomplete or only partially functional"
        echo "selection of components."

        read abort

        echo ""
        echo ""
    fi

fi

#
#Check for existing installations & processes
#

if [ "$INSTALLINGADB" -gt 0 ]
    then

    echo "Checking for existing installations and/or instances of Virtuoso..."

#Various tests
    ps -e | grep virtuoso | grep -v grep  \
        && procs="running processes; "

    netstat -an | grep "111[12].*LIST" > /dev/null \
        && listener="default ports in use; "

    ls -1 | grep 'demo' > /dev/null      \
        && demodir="existing demo database directory; "

    ls -1 | grep 'bin' > /dev/null \
        && ls -1 bin| grep 'virtuoso' > /dev/null  \
        && existexec="existing executables in target; "

    [ -d db ] \
        && ls -1 db | grep virtuoso.ini > /dev/null   \
        && dbini="existing ini file in target; "

    env | egrep -i "(UDBCINI|ODBCINI|ODBCINSTINI)=" > /dev/null \
        && envvars="suspicious environment variables"

#Build test-result string
    INSTANCE="${procs}${listener}${demodir}${existexec}${dbini}${envvars}"

    if [ "z$INSTANCE" != "z" ]
        then

            echo "** Alert: [$INSTANCE]"
            echo ""
            echo "** Warning: You have Virtuoso previously installed on this machine."
            echo "** "
            echo "** To avoid existing data being overwritten or lost, you may press ^C to"
            echo "** exit this installer, terminate and remove the existing installation, or"
            echo "** proceed taking care to avoid port-numbers and passwords used by existing"
            echo "** instances where flagged below, by pressing Enter."

            echo ""
            read abort

    else
        echo "none found"
    fi

#
#Copy existing senstive files out of the way
#
    rm -r .installer-backups            2> /dev/null
    mkdir .installer-backups            2> /dev/null

    echo "Preserving any existing configurations..."

    find db demo bin -name \*.ini -o -name \*.lic -follow -print \
        2> /dev/null | cpio -pdvu .installer-backups

    DOINGABACKUP=0
    # Check whether we did any backups
    find .installer-backups -type f | wc -l | grep '^[  ]*0' > /dev/null || DOINGABACKUP=1

    #
    #Handle license file
    #

    echo ""
    echo ""
    echo "Checking licensing..."

    if [ $INSTALLINGADB -gt 0 ] 
        then

        licfiledef="./virtuoso.lic"

        for i in demo/virtuoso.lic db/virtuoso.lic ./virtuoso.lic
          do
          ( [ -f $i ] ) > /dev/null && licfiledef=$i
        done

        echo ""

        echo 'A license file is required for the Virtuoso Server Instance(s) to'
        echo 'start. Please enter the full path and name of a valid Virtuoso'
        echo "license file [$licfiledef]:"

        overwritewarn

        read licfile

        if [ -z "$licfile" ]
            then
            licfile=$licfiledef
        fi

    fi

#
#Ask first what database wanted on startup, actually start it later
#

    if [ -f $licfile -a \! -z "$licfile" -a $INSTALLINGADB ]
        then
        tostart=2

        echo ''
        echo 'Which Virtuoso server instance would you like to be started after'
        echo 'the installation?'

        overwritewarn

        echo ''
        echo ' 1) Default Database'
        echo ' 2) Demo Database (includes documentation and tutorial)'
        echo ' 3) None'
        echo ""
        echo "Please enter your choice [$tostart] : $echo_c"
        read tostart

        if [ -z "$tostart" ]
            then
            tostart=2
        fi
    fi

fi

#
#Process JDBC/JDK requirements
#

if [ $INSTALLINGCLIENT -gt 0 ]
    then
    echo ""
    echo "Setting up CLASSPATH for Java connectivity."

    overwritewarn

    echo "You can choose between the following Java JDK versions:"
    echo ""
    echo ' 1) JDK 1.1.x'
    echo ' 2) JDK 1.2.x'
    echo ' 3) JDK 1.3.x'
    echo ""
    echo "Please enter your choice [2] : $echo_c"
    read jdbcver
    case $jdbcver in
        1)
            JDKVERSION="jdk1.1"
            ;;
        *)
            JDKVERSION="jdk1.2"
            ;;
    esac
fi

#
#Unpack the tarballs
#

for f in $FILES
  do
  echo Unpacking $f
  $UNZIP -c < $f | tar xvf - | sed -e 's/^x \([^,]*\),.*$/\1/' | tee virt-$f.contents.$$
done

echo "...done"
echo ""

#
#Now copy the license into place
#

if [ $INSTALLINGADB -gt 0 ]
    then

    if [ \! -f "$licfile" ]
        then
        echo ""
        echo "**"
        echo "**  WARNING: The license file entered is not valid or does not exist."
        echo "**   You will need to copy a valid license file manually "
        echo "**   to the bin/ subdirectory after the Virtuoso installation completes."
        echo ""
    else
      # Copy the license only in places where it exists
        if [ \! -f bin/virtuoso.lic ]
            then 
            echo ""
            echo "* Copying the $licfile to bin/virtuoso.lic"
            cp -f "$licfile" bin/virtuoso.lic
        fi
  
    echo ''
    fi

fi

if [ "$INSTALLINGADB" -gt 0 ]
    then

    #Select a server binary, results in $SERVER

    chooseserver

    #Point bin/virtuoso at a valid executable

    cd "${PREFIX}"/bin
    [ -f virtuoso ] && rm virtuoso
    ln -s "$SERVER" virtuoso
    cd "$PREFIX"

    INSTALLINGJAVA=""
    echo $SERVER | grep -i java > /dev/null && INSTALLINGJAVA="1"
    if [ ! -z "$INSTALLINGJAVA" ]
  then
        findcomponent "$SERVER" "libjvm" "the java runtime library"
        JAVA_DIR=$RETVALUE
    fi

    echo "$SERVER" | grep -i mono > /dev/null
    if [ $? = 0 ] 
        then 
        echo ""
        echo "Please enter the full path to the Mono installation directory or"
        echo "press Enter to use the default ($PREFIX/mono):"
        read monolocation 
    fi
    

    if [ -z "$monolocation" ]
        then  
        MONOPATH="$PREFIX/mono"
    fi
    MONO_DIR="$MONOPATH/lib"

    whichperl
    if [ -z "$RETVALUE" ]
        then
        PERL_LIB=
    else  
        PERL_LIB="$RETVALUE"
        LIBPATH="$PERL_LIB:$LIBPATH"
        SHLIB_PATH="$PERL_LIB:$SHLIB_PATH"
        LD_LIBRARY_PATH="$PERL_LIB:$LD_LIBRARY_PATH"
        PERL5LIB=`perl -e 'print join(":",@INC) . "\n"'`
        export LIBPATH SHLIB_PATH LD_LIBRARY_PATH PERL5LIB
    fi
    

        # Find whether we prefer the .so or .a version only if required
    drivers=`find "$PREFIX/lib" -name virtodbcu_r.so\* -o -name virtodbc\*.a`
    driver=`echo $drivers | xargs -n 1 echo | grep '\.so' | head -1`
    [ -z "$driver" ] && driver=`echo $drivers | xargs -n 1 echo | head -1`

    opldrivers=`find "$PREFIX/lib" -name oplodbc.so\* -o -name oplodbc\*.a`
    opldriver=`echo $opldrivers | xargs -n 1 echo | grep '\.so' | head -1`
    [ -z "$opldriver" ] && opldriver=`echo $opldrivers | xargs -n 1 echo | head -1`
    
    opludrivers=`find "$PREFIX/lib" -name oplodbcu.so\* -o -name oplodbcu\*.a`
    opludriver=`echo $opludrivers | xargs -n 1 echo | grep '\.so' | head -1`
    [ -z "$opludriver" ] && opludriver=`echo $opludrivers | xargs -n 1 echo | head -1`

        # Need to escape the / and spaces and dashes in the PWD
    p=`echo "$PREFIX"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    d=`echo "$driver"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    ja=`echo "$JAVA_DIR"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    md=`echo "$MONO_DIR"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    mp=`echo "$MONOPATH"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    pl=`echo "$PERL_LIB"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    od=`echo "$opldriver"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    odu=`echo "$opludriver"| sed -e 's#/#\\\/#g; s# #\\ #g; s#-#\\-#g'`
    JAVA_DIR=$ja
    MONO_DIR=$md
    MONOPATH=$mp
    PERL_LIB=$pl

    echo "Setting up initial values in scripts and config files"

    for f in `find bin/*.sh bin/*.ini bin/*.sample *.sh *.csh db/*.ini demo/*.ini initd* custom/hosting/*/* -type f ! -name \*.db ! -name install.sh | xargs egrep -li 'TOPDIRREPLACEME|DRIVERREPLACEME|DRIVEROPLREPLACEME|DRIVEROPLUREPLACEME|HOSTNAMEREPLACEME|PERLLIBREPLACEME|MONODIRREPLACEME|MONOPATHREPLACEME|JAVALOCATIONREPLACEME' 2> /dev/null`
      do
      if [ -f $f ]
          then
          echo Setting up file $f
              # replace the top-level directories
          sed -e "s/TOPDIRREPLACEME/\"$p\"/g" $f > $f.new && mv $f.new $f
              # replace the odbc driver path
          sed -e "s/DRIVERREPLACEME/$d/g" $f > $f.new && mv $f.new $f
          sed -e "s/DRIVEROPLREPLACEME/$od/g" $f > $f.new && mv $f.new $f
          sed -e "s/DRIVEROPLUREPLACEME/$odu/g" $f > $f.new && mv $f.new $f
              # replace the LD_LIBRARY_PATH bits
          sed -e "s/JAVALOCATIONREPLACEME/\"$JAVA_DIR\"/g" $f > $f.new && mv $f.new $f
              # replace the Mono path bit as well
          sed -e "s/MONODIRREPLACEME/\"$MONO_DIR\"/g" $f > $f.new && mv $f.new $f
          sed -e "s/MONOPATHREPLACEME/\"$MONOPATH\"/g" $f > $f.new && mv $f.new $f

        #Replace the replication host in {virtuoso,demo}.ini
          sed -e "s/HOSTNAMEREPLACEME/\"$HOSTNAME\"/g" $f > $f.new && mv $f.new $f
        #replace the perl lib
          sed -e "s/PERLLIBREPLACEME/\"$PERL_LIB\"/g" $f > $f.new && mv $f.new $f
          sed -e "s/^PERLSAFEREPLACEME/$PERLSAFE/g" $f > $f.new && mv $f.new $f
      fi
    done
    echo "Adding symlink for sample app"
    ln -s  "$PREFIX/vsp/PortalCS" "$PREFIX/vsp/PortalCS/"'~'

    #Restore backups if need be
    #test for presence of backed-up configs

    if [ "$DOINGABACKUP" -gt 0 ]
        then
        echo ""
        echo "Restoring configuration files from backup..."
        cd .installer-backups
        for i in `find . -type f -print`
          do
          #rename the one from the tarball to *.default
          [ -f ../$i ] && mv ../$i ../${i}.default
          echo $i
    rm ../$i 2> /dev/null
          ln $i ../$i
        done
        cd "$PREFIX"
  echo "done."
  echo ""
  echo "Your previous databases and configuration files have been"
  echo "restored from backup."
  echo "The default versions of these files have been renamed"
  echo "to *.default for preservation."
  echo ""
  echo "If everything proceeds successfully below, you might want"
  echo "to remove the .installer-backups/ subdirectory for tidiness."
  echo ""
  echo "Please press Return to continue."
  read junk
  echo ""
    else

        mv demo/demo.db-default demo/demo.db
    #
    #Read in port values
    #
        echo ""
        echo ""
        echo "Configuring port numbers for default and demo databases"
        echo ""
        echo "Please enter a port on which to run the default database"
        echo "SQL listener (default 1111):"

        overwritewarn

        read DBSQLPORT
  [ -z "$DBSQLPORT" ] && DBSQLPORT=1111

        echo ""
        echo "Please enter a port on which to run the default database"
  echo "HTTP listener (default 8889):"

        overwritewarn
  read DBHTTPPORT
  [ -z "$DBHTTPPORT" ] && DBHTTPPORT=8889
        
        # Zero configuration options
        echo ""
        echo "Please enter a name for default database Zero Configuration"
  echo "(default \"Virtuoso Universal Server at $HOSTNAME ($DBSQLPORT)\"):"

        read ZNAME
        [ -z "$ZNAME" ] && ZNAME="Virtuoso Universal Server at $HOSTNAME ($DBSQLPORT)"

        echo "Please enter a user name for default database Zero Configuration"
  echo "(default \"dba\"):"

        read ZDSN
        [ -z "$ZDSN" ] && ZDSN="dba"

        # Demo DB setup
        echo ""
        echo "Please enter a port on which to run the demo database"
  echo "SQL listener (default 1112):"
        overwritewarn

  read DEMOSQLPORT
  [ -z "$DEMOSQLPORT" ] && DEMOSQLPORT=1112

        echo ""
        echo "Please enter a port on which to run the demo database"
  echo "HTTP listener (default 8890):"
        overwritewarn

  read DEMOHTTPPORT
  [ -z "$DEMOHTTPPORT" ] && DEMOHTTPPORT=8890

        # Zero configuration options
        echo ""
        echo "Please enter a name for demonstration database Zero Configuration"
  echo "(default \"Virtuoso Universal Server (demonstration) at $HOSTNAME ($DEMOSQLPORT)\"):"

        read DZNAME
        [ -z "$DZNAME" ] && DZNAME="Virtuoso Universal Server (demonstration) at $HOSTNAME ($DEMOSQLPORT)"

        echo ""
        echo "Please enter a user name for demonstration database Zero Configuration"
  echo "(default demo):"

        read DZDSN
        [ -z "$DZDSN" ] && DZDSN="demo"

    #URIQA

    URIQASTRMAIN="$HOSTNAME:$DBHTTPPORT"
    URIQASTRDEMO="$HOSTNAME:$DEMOHTTPPORT"
	

    #mangle the ini files

        if [ ! -f db/virtuoso.ini ]
            then
            sed "s/DBHTTPPORT/$DBHTTPPORT/; s/DBSQLPORT/$DBSQLPORT/; s/ZNAME/$ZNAME/; s/ZDSN/UID=$ZDSN;PWD=/; s/URIQAREPLACEME/$URIQASTRMAIN/" db/virtuoso.ini.sample > db/virtuoso.ini
        fi

        if [ ! -f demo/demo.ini ]
            then
      sed "s/DEMOHTTPPORT/$DEMOHTTPPORT/; s/DEMOSQLPORT/$DEMOSQLPORT/; s/ZNAME/$DZNAME/; s/ZDSN/UID=$DZDSN;PWD=/; s/URIQAREPLACEME/$URIQASTRDEMO/" demo/demo.ini.sample > demo/demo.ini
        fi

        if [ ! -f bin/odbc.ini ]
            then
      sed "s/DEMOSQLPORT/$DEMOSQLPORT/; s/DBSQLPORT/$DBSQLPORT/" bin/odbc.ini.sample > odbc.ini.temp
            mv odbc.ini.temp bin/odbc.ini
        fi
        
        if [ -f mono/etc/mono/machine.config ]
            then
            sed "s/PORTREPLACE/$DEMOSQLPORT/" mono/etc/mono/machine.config > machine.config.temp
            mv machine.config.temp mono/etc/mono/machine.config 
        fi
        
        if [ -f mono/lib/mono/machine.config ]
            then
            sed "s/PORTREPLACE/$DEMOSQLPORT/" mono/lib/mono/machine.config > machine.config.temp
            mv machine.config.temp mono/lib/mono/machine.config
        fi
        
        if [ -f vsp/PortalCS/Web.config ]
            then
            sed "s/PORTREPLACE/$DEMOSQLPORT/" vsp/PortalCS/Web.config > Web.config.temp
            mv Web.config.temp vsp/PortalCS/Web.config 
        fi

        if [ -f virtuoso-enterprise.sh ]
            then
            sed "s/JDKVERSIONREPLACE/$JDKVERSION/" virtuoso-enterprise.sh > v
            mv v virtuoso-enterprise.sh
        fi

        if [ -f virtuoso-enterprise.csh ]
            then
            sed "s/JDKVERSIONREPLACE/$JDKVERSION/" virtuoso-enterprise.csh > v
            mv v virtuoso-enterprise.csh
        fi

  #Change passwords
  #starting with the db instance

  cd "$PREFIX"

  echo ""
  echo "Changing passwords for the Database Universal Server"

  chpasswd "dba" "Default Database Server" "SQL Administrative"
  dbapw1="$pw1"
  dbapw2="$pw2"
  
  chpasswd "dav" "Default Database" "WebDAV administrative" 
  davpw1=$pw1
  davpw2=$pw2

  echo ""
  echo "Please wait while the passwords are changed:"

  cd db
  ../bin/virtuoso-iodbc-t +pwdold dba +pwddba "$dbapw1" \
                +pwddav "$davpw1" -f +wait > /dev/null 2>&1

        if [ $? -gt 0 ]
        then
            echo '** Problems were encountered starting virtuoso to change the password.'
        fi
 
  vadinstall "OpenLink Dataspace Framework" "../vad/ods_framework_dav.vad" "Default"
  vadinstall "OpenLink BPEL Process Manager" "../vad/bpel_dav.vad" "Default"
  vadinstall "Virtuoso Conductor" "../vad/conductor_dav.vad" "Default"
  shutdown_server $DBSQLPORT
  
  cd ..

  # now do the same for the demo instance

  echo ""
  echo "Changing passwords for the Demo Universal Server"

  cd "$PREFIX"

  chpasswd "dba" "Demonstration Database Server" "SQL Administrative"
  dbapw1=$pw1

  chpasswd "dav" "Demonstration Database Server" "WebDAV Administrative"
  davpw1=$pw1

  echo ""
  echo "Please wait while the passwords are changed:"

  cd demo
  ../bin/virtuoso-iodbc-t -c demo +pwdold dba +pwddba "$dbapw1" \
                +pwddav "$davpw1" -f +wait > /dev/null 2>&1
        if [ $? -gt 0 ]
        then
            echo '** Problems were encountered starting virtuoso to change the password.'
        fi
  
  vadinstall "OpenLink Dataspace Framework" "../vad/ods_framework_dav.vad"
  vadinstall "OpenLink BPEL Process Manager" "../vad/bpel_dav.vad"
  vadinstall "Virtuoso Conductor" "../vad/conductor_dav.vad"
  shutdown_server $DEMOSQLPORT

  cd "$PREFIX"

  echo "done."

    fi

    if [ $DOINGABACKUP -eq 0 ]
    then
        enable_plugins
    fi
fi

#
#Handle ownership & permissions
#
#user=`id -u`
user="$USER"
if [ -z "$user" ]
then 
  user=`id -u` 2> /dev/null
fi
if [ -z "$user" ]
then 
  user=`id | tr '()' '  ' | awk '{print $2}' `
fi
group=`groups | cut -d" " -f1`

echo ""
echo "Please enter the username:group for the user and group who should own"
echo "files in this installation (default: $user:$group) :"

read owner

if [ -z "$owner" ]
then
  owner="$user:$group"
fi

echo "Setting file-ownership to $owner and fixing permissions:"

cat virt-*.contents.$$ | grep -v 'demo/demo.db-default' | while read f
do
  chown $owner "$f"
  chmod og=rX "$f"
done

for ext in '\.csh$' '\.sh$' '/mcs$'
do
  cat virt-*.contents.$$ | grep -v 'demo/demo.db-default' | grep -v ' ' | grep "$ext" | while read f; do chmod 750 "$f"; done
done

rm virt-*.contents.$$

checkgclib $SERVER

#Handle presence of the GRSecurity/PAX kernel

chpaxfiles=`ls /{,usr/}{,s}bin/chpax 2> /dev/null`
matching=`echo "$chpaxfiles" | grep -i "chpax" > /dev/null`

if [ \( "$ONLINUX" -gt 0 \) -a \( \! -z "$matching" \) ]
    then
    echo ""
    echo "You have the chpax utility installed, which normally accompanies"
    echo "the kernel PAX security enhancement patch. If you have this, or the"
    echo "GRSecurity patch installed, we need to run chpax -p to trust the"
    echo "Virtuoso executables (particularly the Mono- and Java-hosting "
    echo "enhanced) servers."
    echo ""
    echo "Enter Yes below if you wish to run the chapax utility now:"
    echo ""

    read RunChPax

    if [ "$RunChPax" -eq "Yes" ]
        then
        chpax -p bin/virtuoso* 2> /dev/null
    fi
fi

#
#Start a database if agreed earlier
#

spiel() {

    dbtype=$1
    sqlport=$2
    httpport=$3

    echo "In order to use the virtuoso executables, you should execute one of"
    echo "the following commands, depending on your current shell:"
    echo ""
    echo "    . ./virtuoso-enterprise.sh"
    echo "for sh, ksh, bash, zsh etc, or"
    echo ""
    echo "    source virtuoso-enterprise.csh"
    echo ""
    echo "for csh and tcsh shells."

    env | grep 'SHELL=.*csh' > /dev/null

    if [ $? -eq 0 ] 
        then
        echo "(The latter option looks more likely.)"
    else
        echo "(The former option looks more likely.)"
    fi

    echo ""

    echo "The $dbtype database (situated in ${PREFIX}/demo) is now"
    echo "accessible for SQL access using the "isql" utility at port"
    echo "$sqlport and HTTP/WebDAV access at port $httpport."
    echo ""
    echo "You may use the isql-iodbc utility with either the 'Local"
    echo "Virtuoso' or 'Local Virtuoso Demo' default ODBC Data Source"
    echo "Names (DSNs). "
    echo "You may also point your browser at <http://${HOSTNAME}:${httpport}/>"
    echo "to use the HTML-based Admin Interface."
    echo ""
}

unhappyspiel() {
    dbtype=$1
    location=$2
    startcmd=$3

    echo "Unfortunately there seems to have been a problem starting the"
    echo "$dbtype server. The last 10 lines of the appropriate log-file are:"

    #quote the log-file so it stands out
    tail $location/*.log | sed 's/^/ | /'

    echo "You should fix this error and then execute one or other of the"
    echo "following pairs of commands, depending on your shell:"

    echo ". ./virtuoso-enterprise.sh"
    echo "$startcmd"

    echo "or"
    echo ". ./virtuoso-enterprise.csh"
    echo "$startcmd"

    echo "This will complete the installation with a running database instance."

}

echo ""

if [ $INSTALLINGADB -gt 0 ]
    then

    case $tostart in
        1)
            echo "*"
            echo "* Starting Virtuoso with a default (empty) database."
            echo "*"
            . ./virtuoso-enterprise.sh
            ./bin/virtuoso-start.sh
            if [ $? != 0 ]
                then
                unhappyspiel "default" "db/" "virtuoso-start.sh"
            else
                spiel "default" "$DBSQLPORT" "$DBHTTPPORT"
            fi
            ;;
        2)
            echo "*"
            echo "* Starting Virtuoso with the Demo database loaded."
            echo "*"
            . ./virtuoso-enterprise.sh
            ./bin/demo-start.sh 
            if [ $? != 0 ] 
                then
                unhappyspiel "demonstration" "demo/" "demo-start.sh"
            else
                spiel "demonstration" "$DEMOSQLPORT" "$DEMOHTTPPORT"
            fi
            ;;

        3)
            echo "In order to use the virtuoso executables, you must execute one of"
            echo "the following commands, depending on your current shell:"
            echo "    . ./virtuoso-enterprise.sh"
            echo "for sh, ksh, bash, zsh etc, or"
            echo "    source virtuoso-enterprise.csh"
            echo "for csh and tcsh shells."

            env | grep 'SHELL=.*csh' > /dev/null

            if [ $? -eq 0 ] 
                then
                echo "(The latter option looks more likely.)"
            else
                echo "(The former option looks more likely.)"
            fi

            echo ""

            echo "To start the Virtuoso Universal Server database instance, use the command"
            echo "   virtuoso-start.sh"
            echo "and virtuoso-stop.sh to stop it."
            echo "To start the Virtuoso Universal Server demonstration instance, use the command"
            echo "   demo-start.sh"
            echo "and demo-stop.sh to stop it."

            ;;

        *)
            echo "The Virtuoso Installer has not offered to start a database as"
            echo "no license key was found. Evaluation users should receive"
            echo "one via e-mail; in case of difficulties please contact "
            echo "<Technical.Support@openlinksw.com>."
            ;;
    esac

fi

#
#Bye bye
#

echo ""
echo "Your Virtuoso installation is now complete."
echo ""
